package com.brilliance.isc.transfer.transaction;

import com.brilliance.mda.runtime.mda.util.Systems;
import com.brilliance.isc.bo.lg.HitpopStoreBo;
import com.brilliance.isc.transfer.IBaseStructMapper;
import com.brilliance.isc.transfer.model.*;
import com.brilliance.isc.vo.lg.HitpopStoreVo;
import com.brilliance.isc.vo.model.CbbVo;
import com.brilliance.isc.vo.model.Giacbs;
import org.mapstruct.AfterMapping;
import org.mapstruct.Mapper;
import org.mapstruct.MappingTarget;
import org.mapstruct.Mappings;
import org.mapstruct.factory.Mappers;


/**
 * Kitopn Bo、Vo之间的转换处理类。如果有很复杂的函数，则可以用适配器模式重写
 */
@Mapper(componentModel = "spring",uses = {
        IGctMapper.class, IGitMapper.class,
        INitMapper.class, INctMapper.class, IOtherModMapper.class}
        )
public interface HitpopStructMapper extends IBaseStructMapper<HitpopStoreBo, HitpopStoreVo> {

    /**
     * 此行语句一定要写，StructMapper需要
     */
    HitpopStructMapper INSTANCE = Mappers.getMapper(HitpopStructMapper.class);

    /**
     * Vo转为Bo
     *
     * @param vo
     * @return 返回业务Bo
     */
    @Override
    HitpopStoreBo voTransferToBo(HitpopStoreVo vo);

    /**
     * Vo追加更新至Bo
     * 忽略oldgrp的复制
     * @param vo
     * @param bo
     */
    @Override
    @Mappings({
//            @Mapping(target = "oldgidgrp",ignore = true)
    })
    void voUpdateToBo(HitpopStoreVo vo, @MappingTarget HitpopStoreBo bo);

    @AfterMapping
    default void afterUpdate(HitpopStoreVo vo, @MappingTarget HitpopStoreBo bo){
        Giacbs cbs = vo.getNidgrp().getCbs();
        if(cbs == null) return;
        CbbVo max = cbs.getMax();
        if(max == null) return;
        bo.getNidgrp().setAmtwrdtxtz(Systems.moneycnsmalltobig(String.valueOf(max.getAmt())));
        bo.getNidgrp().setFmtamtz(String.valueOf(max.getAmt()));
    }

    /**
     * Bo转为Vo
     *
     * @param bo
     * @return 返回Vo
     */
    @Override
    HitpopStoreVo boTransferToVo(HitpopStoreBo bo);

    /**
     * bo追加更新至Bo
     *
     * @param bo
     * @param vo
     */
    @Override
    void boUpdateToVo(HitpopStoreBo bo, @MappingTarget HitpopStoreVo vo);
}
